import uuid

from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey, Numeric, UniqueConstraint, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .inventory import Inventory
    from .modifier import Modifier


@reg.mapped_as_dataclass(unsafe_hash=True)
class InventoryModifier:
    __tablename__ = "inventory_modifiers"
    __table_args__ = (UniqueConstraint("inventory_id", "modifier_id"),)

    id: Mapped[uuid.UUID] = mapped_column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), insert_default=uuid.uuid4
    )
    inventory_id: Mapped[uuid.UUID] = mapped_column(
        "inventory_id", UUID(as_uuid=True), ForeignKey("inventories.id"), nullable=False
    )
    modifier_id: Mapped[uuid.UUID] = mapped_column(
        "modifier_id", UUID(as_uuid=True), ForeignKey("modifiers.id"), nullable=False
    )
    price: Mapped[Decimal] = mapped_column("price", Numeric(precision=15, scale=2), nullable=False)

    inventory: Mapped["Inventory"] = relationship(back_populates="modifiers")
    modifier: Mapped["Modifier"] = relationship("Modifier")

    def __init__(self, inventory_id: uuid.UUID | None, modifier_id: uuid.UUID | None, price: Decimal):
        if inventory_id is not None:
            self.inventory_id = inventory_id
        if modifier_id is not None:
            self.modifier_id = modifier_id
        self.price = price
