import uuid

from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey, Integer, Numeric, UniqueConstraint, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .settle_option import SettleOption
    from .voucher import Voucher


@reg.mapped_as_dataclass(unsafe_hash=True)
class Settlement:
    __tablename__ = "settlements"
    __table_args__ = (UniqueConstraint("voucher_id", "settled"),)

    id: Mapped[uuid.UUID] = mapped_column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), insert_default=uuid.uuid4
    )
    voucher_id: Mapped[uuid.UUID] = mapped_column(
        "voucher_id",
        UUID(as_uuid=True),
        ForeignKey("vouchers.id"),
        nullable=False,
        index=True,
    )
    settled: Mapped[int] = mapped_column("settled", Integer, ForeignKey("settle_options.id"), nullable=False)
    amount: Mapped[Decimal] = mapped_column("amount", Numeric(precision=15, scale=2), nullable=False)

    settle_option: Mapped["SettleOption"] = relationship("SettleOption")
    voucher: Mapped["Voucher"] = relationship(back_populates="settlements")

    def __init__(self, voucher_id=None, settled=None, amount=None, id_=None):
        self.id = id_
        self.voucher_id = voucher_id
        self.settled = settled
        self.amount = amount
