import uuid

from datetime import datetime

from pydantic import BaseModel, ConfigDict, Field, field_serializer

from . import to_camel
from .section import SectionLink


class DeviceIn(BaseModel):
    name: str = Field(..., min_length=1)
    enabled: bool
    section: SectionLink
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class Device(DeviceIn):
    id_: uuid.UUID
    creation_date: datetime
    last_user: str
    last_date: datetime | None = None
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_serializer("creation_date")
    def serialize_creation_date(self, value: datetime, _info):
        return value.strftime("%d-%b-%Y %H:%M")

    @field_serializer("last_date")
    def serialize_last_date(self, value: datetime | None, _info):
        return None if value is None else value.strftime("%d-%b-%Y %H:%M")


class DeviceLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: str | None = None
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)
