import uuid

from datetime import date, datetime

from pydantic import BaseModel, ConfigDict, Field, field_serializer, field_validator

from . import to_camel


class GuestBookIn(BaseModel):
    name: str
    phone: str
    address: str | None = None
    pax: int = Field(ge=0)
    date_: datetime

    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value):
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("date_")
    def serialize_date(self, value: datetime, _info):
        return value.strftime("%d-%b-%Y %H:%M")


class GuestBook(GuestBookIn):
    id_: uuid.UUID

    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class GuestBookListItem(BaseModel):
    id_: uuid.UUID
    serial: int
    name: str
    phone: str
    pax: int
    date_: datetime
    status: str | None = None
    table_id: uuid.UUID | None = None
    voucher_id: uuid.UUID | None = None
    table_name: str | None = None

    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value):
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @field_serializer("date_")
    def serialize_date(self, value: datetime, _info):
        return value.strftime("%d-%b-%Y %H:%M")


class GuestBookList(BaseModel):
    date_: date
    list: list[GuestBookListItem]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, _info):
        return value.strftime("%d-%b-%Y")
