import uuid

from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field

from . import to_camel
from .modifier_category import ModifierCategoryLink


class ModifierIn(BaseModel):
    name: str = Field(..., min_length=1)
    show_in_bill: bool
    price: Decimal = Field(ge=0, default=0)
    is_active: bool
    modifier_category: ModifierCategoryLink
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class Modifier(ModifierIn):
    id_: uuid.UUID


class ModifierLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: str | None = None
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)


class ModifierBlank(BaseModel):
    name: str
    show_in_bill: bool
    price: Decimal
    is_active: bool
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)
