import uuid

from datetime import date, datetime
from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field, field_serializer, field_validator

from . import to_camel
from .menu_category import MenuCategoryLink
from .sale_category import SaleCategoryLink


class ProductIn(BaseModel):
    name: str = Field(..., min_length=1)
    units: str
    menu_category: MenuCategoryLink = Field(...)
    sale_category: SaleCategoryLink = Field(...)
    price: Decimal  # = Field(ge=0, default=0)
    has_happy_hour: bool
    is_not_available: bool
    quantity: Decimal = Field(ge=0, default=0)
    is_active: bool
    sort_order: int
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class Product(ProductIn):
    id_: uuid.UUID
    version_id: uuid.UUID
    valid_from: date | None = None
    valid_till: date | None = None
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("valid_from", mode="before")
    @classmethod
    def parse_valid_from(cls, value):
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("valid_from")
    def serialize_valid_from(self, value: date | None, _info):
        return None if value is None else value.strftime("%d-%b-%Y")

    @field_validator("valid_till", mode="before")
    @classmethod
    def parse_valid_till(cls, value):
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("valid_till")
    def serialize_valid_till(self, value: date | None, _info):
        return None if value is None else value.strftime("%d-%b-%Y")


class ProductBlank(BaseModel):
    name: str
    units: str
    price: Decimal
    has_happy_hour: bool
    is_not_available: bool
    quantity: Decimal
    is_active: bool
    sort_order: int
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)
