import uuid

from barker.schemas.permission import PermissionItem
from pydantic import BaseModel, ConfigDict, Field

from . import to_camel


class RoleIn(BaseModel):
    name: str = Field(..., min_length=1)
    permissions: list[PermissionItem]
    model_config = ConfigDict(str_strip_whitespace=True)


class Role(RoleIn):
    id_: uuid.UUID
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class RoleBlank(RoleIn):
    name: str
    model_config = ConfigDict(str_strip_whitespace=True)


class RoleList(BaseModel):
    id_: uuid.UUID
    name: str
    permissions: list[str]
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True, str_strip_whitespace=True)


class RoleItem(BaseModel):
    id_: uuid.UUID
    name: str
    enabled: bool
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)
