import uuid

from decimal import Decimal

from pydantic import BaseModel, ConfigDict, Field

from . import to_camel
from .tax import TaxLink


class SaleCategoryIn(BaseModel):
    name: str = Field(..., min_length=1)
    discount_limit: Decimal = Field(ge=0, default=0, le=1)
    tax: TaxLink = Field(...)
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class SaleCategory(SaleCategoryIn):
    id_: uuid.UUID
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class SaleCategoryBlank(BaseModel):
    name: str
    discount_limit: Decimal
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class SaleCategoryForDiscount(BaseModel):
    id_: uuid.UUID
    name: str
    discount_limit: Decimal
    discount: Decimal
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class SaleCategoryLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: str | None = None
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)
