import uuid

from datetime import datetime

from barker.schemas import to_camel
from barker.schemas.role import RoleItem
from pydantic import BaseModel, ConfigDict, field_serializer


class UserIn(BaseModel):
    name: str
    password: str
    locked_out: bool
    roles: list[RoleItem]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class User(UserIn):
    id_: uuid.UUID
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class UserMe(UserIn):
    id_: str
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class UserLink(BaseModel):
    id_: uuid.UUID | None = None
    name: str | None = None
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)


class UserList(BaseModel):
    id_: uuid.UUID
    name: str
    locked_out: bool
    roles: list[str]
    last_device: str
    last_date: datetime | None = None

    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_serializer("last_date")
    def serialize_last_date(self, value: datetime | None, _info):
        return None if value is None else value.strftime("%d-%b-%Y %H:%M")
