import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { ProductUpdatesReport } from './product-updates-report';

const url = '/api/product-updates-report';
const serviceName = 'ProductUpdatesReportService';

@Injectable({
  providedIn: 'root',
})
export class ProductUpdatesReportService {
  constructor(
    private http: HttpClient,
    private log: ErrorLoggerService,
  ) {}

  get(startDate: string | null, finishDate: string | null): Observable<ProductUpdatesReport> {
    const options = { params: new HttpParams() };
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return this.http
      .get<ProductUpdatesReport>(url, options)
      .pipe(catchError(this.log.handleError(serviceName, 'get'))) as Observable<ProductUpdatesReport>;
  }
}
