import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';
import { RegimeListResolver } from '../regimes/regime-list-resolver.service';

import { TaxDetailComponent } from './tax-detail/tax-detail.component';
import { TaxListComponent } from './tax-list/tax-list.component';
import { TaxListResolver } from './tax-list-resolver.service';
import { TaxResolver } from './tax-resolver.service';

const taxesRoutes: Routes = [
  {
    path: '',
    component: TaxListComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Taxes',
    },
    resolve: {
      list: TaxListResolver,
    },
  },
  {
    path: 'new',
    component: TaxDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Taxes',
    },
    resolve: {
      item: TaxResolver,
      regimes: RegimeListResolver,
    },
  },
  {
    path: ':id',
    component: TaxDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Taxes',
    },
    resolve: {
      item: TaxResolver,
      regimes: RegimeListResolver,
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(taxesRoutes)],
  exports: [RouterModule],
  providers: [TaxListResolver, TaxResolver],
})
export class TaxesRoutingModule {}
